'''
Gauged - https://github.com/chriso/gauged
Copyright 2014 (c) Chris O'Hara <cohara87@gmail.com>
'''

import gauged, unittest, sys, gc
from ConfigParser import ConfigParser
from test import (TestGauged, TestStructures, TestLRU, TestDriver,
    TestDSN, TestResult)

# Get the list of test drivers
config = ConfigParser()
config.read('test_drivers.cfg')
test_drivers = config.items('drivers')

# Parse command line options
argv = set(sys.argv)
quick_tests = '--quick' in argv    # Use only the first driver (in-memory SQLite)
raise_on_error = '--raise' in argv # Stop and dump a trace when a test driver fails
run_forever = '--forever' in argv  # Run the tests forever (to check for leaks)
drivers_only = '--drivers' in argv # Run driver tests only
verbose = '--verbose' in argv      # Increase test runner verbosity

if quick_tests:
    test_drivers = test_drivers[:1]

def run_tests():

    suite = unittest.TestSuite()

    test_class = driver = None

    # Test each driver in gauged/drivers. We need to dynamically subclass
    # the TestDriver class here - the unittest module doesn't allow us to
    # use __init__ to pass in driver arguments or a driver instance
    for driver, dsn in test_drivers:
        try:
            gauged_instance = gauged.Gauged(dsn)
            # Empty the data store
            gauged_instance.driver.drop_schema()
            gauged_instance.sync()
            gauged_instance.driver.create_schema()
            # Test the driver class
            test_class = type('Test%s' % driver, (TestDriver,), {})
            test_class.driver = gauged_instance.driver
            suite.addTest(unittest.makeSuite(test_class))
            # Test gauged/gauged.py using the driver
            if not drivers_only:
                test_class = type('TestGaugedWith%s' % driver, (TestGauged,), {})
                test_class.driver = gauged_instance.driver
                suite.addTest(unittest.makeSuite(test_class))
        except Exception as e:
            print 'Skipping %s tests (%s). Check test_driver.cfg' % (driver[:-6], str(e).rstrip())
            if raise_on_error:
                raise

    if test_class is None:
        msg = 'No drivers available for unit tests, check configuration in test_driver.cfg'
        raise RuntimeWarning(msg)

    # Test the remaining classes
    if not drivers_only:
        suite.addTest(unittest.makeSuite(TestStructures))
        suite.addTest(unittest.makeSuite(TestLRU))
        suite.addTest(unittest.makeSuite(TestDSN))
        suite.addTest(unittest.makeSuite(TestResult))

    # Setup the test runner
    verbosity = 2 if verbose else 1
    if run_forever:
        verbosity = 0
    test_runner = unittest.TextTestRunner(verbosity=verbosity)

    # Run the tests
    while True:
        result = test_runner.run(suite)
        if result.errors or result.failures:
            exit(1)
        if not run_forever:
            break
        gc.collect()

if __name__ == '__main__':
    run_tests()



'''
Gauged
https://github.com/chriso/gauged (MIT Licensed)
Copyright 2014 (c) Chris O'Hara <cohara87@gmail.com>
'''

class GaugedError(Exception):
    '''All Gauged errors inherit from this class'''

class GaugedAppendOnlyError(GaugedError):
    '''An error that occurs when gauges aren't sent to writer.add() in
    chronological order'''

class GaugedDateRangeError(GaugedError):
    '''An error that occurs when the start & end dates in a date range are
    overlapping'''

class GaugedKeyOverflowError(GaugedError):
    '''An error that occurs when the current driver cannot handle
    a large key'''

class GaugedIntervalSizeError(GaugedError):
    '''Occurs when a time series operation contains too many interval
    steps in the specified date range'''

class GaugedNaNError(GaugedError):
    '''Occurs when you try and write a NaN value'''

class GaugedUseAfterFreeError(GaugedError):
    '''Occurs when a structure that allocates memory is used after it
    has been freed, e.g. outside of its context manager'''

class GaugedVersionMismatchError(GaugedError):
    '''Occurs when the Gauged version does not match the version stored in
    the database'''

class GaugedBlockSizeMismatch(RuntimeWarning):
    '''Occurs when the configured block_size and/or resolution doesn't
    match those stored in the database'''

class GaugedSchemaError(GaugedError):
    '''Occurs when an operation is attempted and no schema can be found'''



'''
Gauged
https://github.com/chriso/gauged (MIT Licensed)
Copyright 2014 (c) Chris O'Hara <cohara87@gmail.com>
'''

from types import DictType
from ..utilities import table_repr, to_datetime

class TimeSeries(object):
    '''A representation of a time series with a fixed interval'''

    def __init__(self, points):
        '''Initialise the time series. `points` is expected to be either a list of
        tuples where each tuple represents a point (timestamp, value), or a dict where
        the keys are timestamps. Timestamps are expected to be in milliseconds'''
        if type(points) == DictType:
            points = points.items()
        self.points = sorted(points)

    @property
    def timestamps(self):
        '''Get all timestamps from the series'''
        return [ point[0] for point in self.points ]

    @property
    def dates(self):
        '''Get all dates from the time series as `datetime` instances'''
        return [ to_datetime(timestamp) for timestamp, _ in self.points ]

    @property
    def values(self):
        '''Get all values from the time series'''
        return [ point[1] for point in self.points ]

    @property
    def interval(self):
        if len(self.points) <= 1:
            return None
        return self.points[1][0] - self.points[0][0]

    def map(self, fn):
        '''Run a map function across all y points in the series'''
        return TimeSeries([ (x, fn(y)) for x, y in self.points ])

    def __abs__(self):
        return TimeSeries([ (x, abs(y)) for x, y in self.points ])

    def __round__(self, n=0):
        return TimeSeries([ (x, round(y, n)) for x, y in self.points ])

    def round(self, n=0):
        # Manual delegation for v2.x
        return self.__round__(n)

    def __add__(self, operand):
        if not isinstance(operand, TimeSeries):
            return TimeSeries([ ( x, y + operand ) for x, y in self.points ])
        lookup = dict(operand.points)
        return TimeSeries([ ( x, y + lookup[x] ) for x, y in self.points if x in lookup ])

    def __iadd__(self, operand):
        if not isinstance(operand, TimeSeries):
            self.points = [ ( x, y + operand ) for x, y in self.points ]
        else:
            lookup = dict(operand.points)
            self.points = [ ( x, y + lookup[x] ) for x, y in self.points if x in lookup ]
        return self

    def __sub__(self, operand):
        if not isinstance(operand, TimeSeries):
            return TimeSeries([ ( x, y - operand ) for x, y in self.points ])
        lookup = dict(operand.points)
        return TimeSeries([ ( x, y - lookup[x] ) for x, y in self.points if x in lookup ])

    def __isub__(self, operand):
        if not isinstance(operand, TimeSeries):
            self.points = [ ( x, y - operand ) for x, y in self.points ]
        else:
            lookup = dict(operand.points)
            self.points = [ ( x, y - lookup[x] ) for x, y in self.points if x in lookup ]
        return self

    def __mul__(self, operand):
        if not isinstance(operand, TimeSeries):
            return TimeSeries([ ( x, y * operand ) for x, y in self.points ])
        lookup = dict(operand.points)
        return TimeSeries([ ( x, y * lookup[x] ) for x, y in self.points if x in lookup ])

    def __imul__(self, operand):
        if not isinstance(operand, TimeSeries):
            self.points = [ ( x, y * operand ) for x, y in self.points ]
        else:
            lookup = dict(operand.points)
            self.points = [ ( x, y * lookup[x] ) for x, y in self.points if x in lookup ]
        return self

    def __div__(self, operand):
        if not isinstance(operand, TimeSeries):
            return TimeSeries([ ( x, float(y) / operand ) for x, y in self.points ])
        lookup = dict(operand.points)
        return TimeSeries([ ( x, float(y) / lookup[x] ) for x, y in self.points if x in lookup ])

    def __idiv__(self, operand):
        if not isinstance(operand, TimeSeries):
            self.points = [ ( x, float(y) / operand ) for x, y in self.points ]
        else:
            lookup = dict(operand.points)
            self.points = [ ( x, float(y) / lookup[x] ) for x, y in self.points if x in lookup ]
        return self

    def __pow__(self, operand):
        if not isinstance(operand, TimeSeries):
            return TimeSeries([ ( x, y ** operand ) for x, y in self.points ])
        lookup = dict(operand.points)
        return TimeSeries([ ( x, y ** lookup[x] ) for x, y in self.points if x in lookup ])

    def __ipow__(self, operand):
        if not isinstance(operand, TimeSeries):
            self.points = [ ( x, y ** operand ) for x, y in self.points ]
        else:
            lookup = dict(operand.points)
            self.points = [ ( x, y ** lookup[x] ) for x, y in self.points if x in lookup ]
        return self

    def __getitem__(self, x):
        return dict(self.points)[x]

    def __iter__(self):
        return iter(self.points)

    def __len__(self):
        return len(self.points)

    def __repr__(self):
        if not len(self.points):
            return 'TimeSeries([])'
        data = {}
        columns = [ 'Value' ]
        rows = []
        for date, value in self.points:
            date = to_datetime(date).isoformat(' ')
            rows.append(date)
            data[date] = { 'Value': value }
        return table_repr(columns, rows, data)



'''
Gauged - https://github.com/chriso/gauged
Copyright 2014 (c) Chris O'Hara <cohara87@gmail.com>
'''

import re
from datetime import datetime
from gauged.results import Statistics, TimeSeries
from .test_case import TestCase

class TestResult(TestCase):
    '''Test result wrappers'''

    def test_statistics_repr(self):
        stat = Statistics(1, 2, 3, 4, 5)
        self.assertEqual(str(stat), 'Statistics(namespace=1, start=2, end=3, ' \
            'data_points=4, byte_count=5)')

    def test_tuple_list_init(self):
        series = TimeSeries([ (1, 2), (3, 4), (5, 6) ])
        self.assertListEqual(series.timestamps, [1, 3, 5])
        self.assertListEqual(series.values, [2, 4, 6])
        self.assertEqual(len(series), 3)

    def test_dict_init(self):
        series = TimeSeries({ 1: 2, 3: 4, 5: 6 })
        self.assertListEqual(series.timestamps, [1, 3, 5])
        self.assertListEqual(series.values, [2, 4, 6])
        self.assertEqual(len(series), 3)

    def test_accessors(self):
        points = [ (1234000, 54), (5678000, 100) ]
        series = TimeSeries(points)
        dates = series.dates
        for date in dates:
            self.assertTrue(isinstance(date, datetime))
        self.assertListEqual(series.dates, [datetime.fromtimestamp(1234),
            datetime.fromtimestamp(5678)])
        self.assertListEqual(series.timestamps, [1234000, 5678000])
        self.assertListEqual(series.values, [54, 100])

    def test_initial_sort(self):
        points = [ (3, 54), (2, 100), (4, 32) ]
        series = TimeSeries(points)
        self.assertListEqual(series.timestamps, [2, 3, 4])
        self.assertListEqual(series.values, [100, 54, 32])

    def test_interval(self):
        series = TimeSeries([])
        self.assertEqual(series.interval, None)
        series = TimeSeries([ (1, 2) ])
        self.assertEqual(series.interval, None)
        series = TimeSeries([ (1, 2), (3, 4) ])
        self.assertEqual(series.interval, 2)

    def test_map(self):
        series = TimeSeries([ (1, 2), (3, 4), (5, 6) ])
        double = series.map(lambda y: y * 2)
        self.assertTrue(isinstance(double, TimeSeries))
        self.assertListEqual([ (1, 4), (3, 8), (5, 12) ], double.points)

    def test_indexing(self):
        series = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        self.assertEqual(series[1], 3)
        self.assertEqual(series[2], 3)
        with self.assertRaises(KeyError):
            series[4]

    def test_iteration(self):
        points = [ (1, 2), (3, 4), (5, 6) ]
        series = TimeSeries(points)
        self.assertListEqual([ s for s in series ], points)

    def test_map_return_type(self):
        series = TimeSeries([ (1, 2), (3, 4), (5, 6) ])
        double = series.map(lambda y: y * 2)
        self.assertTrue(isinstance(double, TimeSeries))
        self.assertListEqual([ (1, 4), (3, 8), (5, 12) ], double.points)

    def test_add(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 1), (1, 1), (2, 1), (3, 1), (4, 1) ])
        c = a + b
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 4), (2, 4), (3, 4) ])
        c = c + 5
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 9), (2, 9), (3, 9) ])

    def test_add_update(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 1), (1, 1), (2, 1), (3, 1), (4, 1) ])
        a += b
        self.assertListEqual(a.points, [ (1, 4), (2, 4), (3, 4) ])
        a += 5
        self.assertListEqual(a.points, [ (1, 9), (2, 9), (3, 9) ])

    def test_sub(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 1), (1, 1), (2, 1), (3, 1), (4, 1) ])
        c = a - b
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 2), (2, 2), (3, 2) ])
        c = c - 1
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 1), (2, 1), (3, 1) ])

    def test_sub_update(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 1), (1, 1), (2, 1), (3, 1), (4, 1) ])
        a -= b
        self.assertListEqual(a.points, [ (1, 2), (2, 2), (3, 2) ])
        a -= 1
        self.assertListEqual(a.points, [ (1, 1), (2, 1), (3, 1) ])

    def test_mul(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 2), (1, 3), (2, 2), (3, 2), (4, 1) ])
        c = a * b
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 9), (2, 6), (3, 6) ])
        c = c * 2
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 18), (2, 12), (3, 12) ])

    def test_mul_update(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 2), (1, 3), (2, 2), (3, 2), (4, 1) ])
        a *= b
        self.assertListEqual(a.points, [ (1, 9), (2, 6), (3, 6) ])
        a *= 2
        self.assertListEqual(a.points, [ (1, 18), (2, 12), (3, 12) ])

    def test_div(self):
        a = TimeSeries([ (1, 3), (2, 4), (3, 3) ])
        b = TimeSeries([ (0, 2), (1, 3), (2, 2), (3, 2), (4, 1) ])
        c = a / b
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 1), (2, 2), (3, 1.5) ])
        c = c / 2
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 0.5), (2, 1), (3, 0.75) ])

    def test_div_update(self):
        a = TimeSeries([ (1, 3), (2, 4), (3, 3) ])
        b = TimeSeries([ (0, 2), (1, 3), (2, 2), (3, 2), (4, 1) ])
        a /= b
        self.assertListEqual(a.points, [ (1, 1), (2, 2), (3, 1.5) ])
        a /= 0.5
        self.assertListEqual(a.points, [ (1, 2), (2, 4), (3, 3) ])

    def test_pow(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 2), (1, 3), (2, 2), (3, 1), (4, 1) ])
        c = a ** b
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 27), (2, 9), (3, 3) ])
        c = c ** 2
        self.assertTrue(isinstance(c, TimeSeries))
        self.assertListEqual(c.points, [ (1, 729), (2, 81), (3, 9) ])

    def test_abs(self):
        a = TimeSeries([ (1, -3), (2, 3.3), (3, -5) ])
        a = abs(a)
        self.assertTrue(isinstance(a, TimeSeries))
        self.assertListEqual(a.values, [ 3, 3.3, 5 ])

    def test_round(self):
        a = TimeSeries([ (1, -0.3), (2, 3.3), (3, 1.1) ])
        a = a.round()
        self.assertTrue(isinstance(a, TimeSeries))
        self.assertListEqual(a.values, [ 0, 3, 1 ])

    def test_pow_update(self):
        a = TimeSeries([ (1, 3), (2, 3), (3, 3) ])
        b = TimeSeries([ (0, 2), (1, 3), (2, 2), (3, 1), (4, 1) ])
        a **= b
        self.assertListEqual(a.points, [ (1, 27), (2, 9), (3, 3) ])
        a **= 2
        self.assertListEqual(a.points, [ (1, 729), (2, 81), (3, 9) ])

    def test_time_series_repr(self):
        a = TimeSeries([ (10000, 500), (20000, 1234), (30000, 12345678) ])
        lines = repr(a).split('\n')
        self.assertEqual(lines[0], '                        Value')
        self.assertTrue(re.match(r'^19(?:69|70)-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:10       500$', lines[1]))
        self.assertTrue(re.match(r'^19(?:69|70)-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:20      1234$', lines[2]))
        self.assertTrue(re.match(r'^19(?:69|70)-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:30  12345678$', lines[3]))
        self.assertEqual(str(TimeSeries([])), 'TimeSeries([])')
